local addon, shared = ...

local string_format = string.format

_SYNTHESIS = {UI={tracker={}}}
local ST = _SYNTHESIS

local ST_GROUP = {}
local ST_UnitDetail = {}
local ST_Active = {}
local ST_ActiveBuff = {}

local ST_UPDATEREQ = false
local lastUpdate = 0
local thisUpdate = 0

local playerID = ""
local buffID = ""

local STActive = false

function ST.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if SynthesisSpy_Settings == nil then
			local scx = math.floor(UIParent:GetWidth()/2)-12
			local scy = math.floor(UIParent:GetHeight()/2)-12
			SynthesisSpy_Settings = {x=scx, y=scy}
		end
		ST.UI.tracker[1].i:SetPoint("TOPLEFT", UIParent, "TOPLEFT", SynthesisSpy_Settings.x, SynthesisSpy_Settings.y)
	end
end

function ST.Event_Buff_Remove(h,u,t)
	local uid = ""
	for k,v in pairs(t) do
		uid = ST_ActiveBuff[k]
		if uid then
			if ST_Active[uid] == u then
				ST_Active[uid] = nil
			end
			ST_ActiveBuff[k] = nil
			lastUpdate = 0
			ST_UPDATEREQ = true
			print(string.format("SYNTH off %s", ST_UnitDetail[u] or "??"))
		end
	end
end

function ST.Event_Buff_Add(h,u,t)
	for k,v in pairs(Inspect.Buff.Detail(u,t)) do
		if v.abilityNew == "A741E6F5A9B514DA7" and ST_GROUP[v.caster] and ST_GROUP[u] then
--			print("SYNTH detected")
--			print(string.format("C: %s (%s)", v.caster, tostring(ST_GROUP[v.caster])))
--			print(string.format("U: %s (%s)", u, tostring(ST_GROUP[u])))
--			if ST_GROUP[v.caster] and ST_GROUP[u] then
				ST_Active[v.caster] = u
				ST_ActiveBuff[k] = v.caster
				lastUpdate = 0
				ST_UPDATEREQ = true
				print(string.format("SYNTH on %s by %s", ST_UnitDetail[u] or "??", ST_UnitDetail[v.caster] or "??"))
--			end
		end
	end
end

local groupID = ""
local uid = 0

function ST.Event_System_Update_Begin(h)
	thisUpdate = Inspect.Time.Real()
	if thisUpdate - lastUpdate > 2 then
		lastUpdate = thisUpdate
		ST_GROUP = {}
		for x=1, 20 do
			groupID = string_format("group%02d", x)
			uid = Inspect.Unit.Lookup(groupID)
			if uid then
				ST_GROUP[uid] = true
				if ST_UnitDetail[uid] == nil then
					local pd = Inspect.Unit.Detail(uid)
					if pd then
						ST_UnitDetail[uid] = pd.name
					end				
				end
			end
		end
		uid = Inspect.Unit.Lookup("player")
		if uid then
			ST_GROUP[uid] = true
		end
		
		for b,c in pairs(ST_ActiveBuff) do
			local t = ST_Active[c]
			if ST_GROUP[c] == nil or ST_GROUP[t] == nil then
				print(string.format("%s or %s no longer in raid. Removing synth.",tostring(c),tostring(t)))
				ST.Event_Buff_Remove(0,t,{[b]=true})
			end
		end
		
		if ST_UPDATEREQ then
			local cnt = 0
			for src,trg in pairs(ST_Active) do
				cnt=cnt+1
				if ST_UnitDetail[src] == nil then
					local pd = Inspect.Unit.Detail(src)
					if pd then
						ST_UnitDetail[src] = pd.name
					end
				end
				ST.UI.tracker[cnt].c:SetText(ST_UnitDetail[src] or src)
				if ST_UnitDetail[trg] == nil then
					local pd = Inspect.Unit.Detail(trg)
					if pd then
						ST_UnitDetail[trg] = pd.name
					end
				end
				ST.UI.tracker[cnt].t:SetText(ST_UnitDetail[trg] or trg)
				if src==playerID or trg==playerID then
					ST.UI.tracker[cnt].c:SetFontColor(0,1,0)
					ST.UI.tracker[cnt].t:SetFontColor(0,1,0)
				else
					ST.UI.tracker[cnt].c:SetFontColor(1,1,1)
					ST.UI.tracker[cnt].t:SetFontColor(1,1,1)
				end
				ST.UI.tracker[cnt].i:SetVisible(true)
			end
			for x=cnt+1, 20 do
				ST.UI.tracker[x].i:SetVisible(false)
			end
			if ST_UnitDetail[src] and ST_UnitDetail[trg] then
				ST_UPDATEREQ = false
			end
		end
	end
end

function ST.SetupUI()
	ST.UI.context = UI.CreateContext(addon.identifier)
	
	for x=1, 20 do
		local icon = UI.CreateFrame("Texture", "ST.UI.icon"..x, ST.UI.context)
		icon:SetVisible(false)
		icon:SetWidth(24)
		icon:SetHeight(24)
		icon:SetTexture("Rift", "magicresist3.dds")

		if x == 1 then
			icon:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
				self.MouseDown = true
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - ST.UI.tracker[1].i:GetLeft()
				self.sy = mouseData.y - ST.UI.tracker[1].i:GetTop()		
			end, "Event.UI.Input.Mouse.Left.Down")

			icon:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
				self.MouseDown = false
				SynthesisSpy_Settings.x = ST.UI.tracker[1].i:GetLeft()
				SynthesisSpy_Settings.y = ST.UI.tracker[1].i:GetTop()
			end, "Event.UI.Input.Mouse.Left.Up")

			icon:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
				if self.MouseDown then
					local nx, ny
					local mouseData = Inspect.Mouse()
					nx = mouseData.x - self.sx
					ny = mouseData.y - self.sy
					ST.UI.tracker[1].i:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
				end
			end, "Event.UI.Input.Mouse.Cursor.Move")
		else
			icon:SetPoint("TOPLEFT", ST.UI.tracker[x-1].i, "BOTTOMLEFT", 0,0)
		end
		
		local c_label = UI.CreateFrame("Text", "ST.UI.c_label"..x, icon)
		c_label:SetPoint("CENTERRIGHT", icon, "CENTERLEFT", 2,0)
		c_label:SetFontSize(16)
		c_label:SetText("C:"..x)
		
		local t_label = UI.CreateFrame("Text", "ST.UI.t_label"..x, icon)
		t_label:SetPoint("CENTERLEFT", icon, "CENTERRIGHT", 2,0)
		t_label:SetFontSize(16)
		t_label:SetText("T:"..x)
		
		ST.UI.tracker[x] = {i=icon, c=c_label, t=t_label}
	
	end
	
	Command.Event.Attach(Event.Buff.Add, ST.Event_Buff_Add, "Event.Buff.Add")
	Command.Event.Attach(Event.Buff.Remove, ST.Event_Buff_Remove, "Event.Buff.Remove")
end

ST.SetupUI()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, ST.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.System.Update.Begin, ST.Event_System_Update_Begin, "Event.System.Update.Begin")

playerID = Inspect.Unit.Lookup("player")

print(string.format("v%s loaded.", addon.toc.Version))